package djmil.cordacheckers;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertDoesNotThrow;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import djmil.cordacheckers.user.HoldingIdentityResolver;

@SpringBootTest
public class HoldingIdentityResolverTests {

    @Autowired
    HoldingIdentityResolver holdingIdentityResolver;

    /*
     * The test is expecting that CSDE static-network-config.json to look like stated below.
     * Important bits: 
     *  - the contents of 'CN', 'OU', 'O' fields
     *  - there shall be only one Custodian
     */
/*
    [
        {
            "x500Name" : "CN=Alice, OU=Player, O=Checkers, L=Zug, C=CH",
            "cpi" : "CordaCheckers"
        },
        {
            "x500Name" : "CN=Bob, OU=Player, O=Checkers, L=Kyiv, C=UA",
            "cpi" : "CordaCheckers"
        },
        {
            "x500Name" : "CN=Charlie, OU=Player, O=Checkers, L=London, C=GB",
            "cpi" : "CordaCheckers"
        },
        {
            "x500Name" : "CN=Kumar, OU=Custodian, O=Checkers, L=Mumbai, C=IN",
            "cpi" : "CordaCheckers"
        },
        {
            "x500Name" : "CN=NotaryRep1, OU=Test Dept, O=R3, L=London, C=GB",
            "cpi" : "NotaryServer",
            "serviceX500Name": "CN=NotaryService, OU=Test Dept, O=R3, L=London, C=GB"
        }
    ]
*/

    @Test
    void testPlayer() {
        assertDoesNotThrow( () -> {
            holdingIdentityResolver.getByUsername("alice");
            holdingIdentityResolver.getByUsername("BOB");
            holdingIdentityResolver.getByUsername("Charlie");
        });

        assertThat(holdingIdentityResolver.getByUsername("SomeRundomName")).isNull();
    }

    @Test
    void testCustodian() {
        assertThat(holdingIdentityResolver.getByUsername("Kumar")).isNull();
        assertThat(holdingIdentityResolver.getCustodian()).isNotNull();
    }
    
}
