package djmil.cordacheckers.gameboard;

import java.time.Duration;
import java.time.Instant;
import java.util.UUID;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import djmil.cordacheckers.contracts.GameCommand;
import djmil.cordacheckers.gamestate.CommitTrx;
import djmil.cordacheckers.gamestate.FlowResponce;
import djmil.cordacheckers.gamestate.GetFlow;
import djmil.cordacheckers.gamestate.View;
import djmil.cordacheckers.gamestate.ViewBuilder;
import djmil.cordacheckers.states.GameBoardState;
import djmil.cordacheckers.states.GameState;
import net.corda.v5.application.flows.ClientRequestBody;
import net.corda.v5.application.flows.ClientStartableFlow;
import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.FlowEngine;
import net.corda.v5.application.marshalling.JsonMarshallingService;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.StateAndRef;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;

public class DrawRequestFlow implements ClientStartableFlow{

     private final static Logger log = LoggerFactory.getLogger(DrawRequestFlow.class);

     @CordaInject
     public JsonMarshallingService jsonMarshallingService;

     @CordaInject
     public MemberLookup memberLookup;

     @CordaInject
     public UtxoLedgerService utxoLedgerService;

     @CordaInject
     public FlowEngine flowEngine;

     @Suspendable
     @Override
     public String call(ClientRequestBody requestBody) {
          SecureHash utxoTrxId = null;

          try {
               final GameCommand command = new GameCommand(GameCommand.Action.DRAW_REQUEST);
               final UUID gameUuid = UUID.fromString(requestBody.getRequestBody());

               final StateAndRef<GameState> currentGameSar = this.flowEngine
                    .subFlow(new GetFlow(gameUuid));
               final GameBoardState currenGameBoardState = (GameBoardState)currentGameSar.getState().getContractState();

               final GameBoardState newGameBoard = new GameBoardState(
                    currenGameBoardState,
                    true); // <<-- draw request

               final UtxoSignedTransaction drawReqTrx = utxoLedgerService.createTransactionBuilder()
                    .addCommand(command)
                    .addInputState(currentGameSar.getRef())
                    .addOutputState(newGameBoard)
                    .addSignatories(newGameBoard.getParticipants())
                    .setNotary(currentGameSar.getState().getNotaryName())
                    .setTimeWindowUntil(Instant.now().plusMillis(Duration.ofDays(1).toMillis()))
                    .toSignedTransaction();

               utxoTrxId = this.flowEngine
                    .subFlow(new CommitTrx(drawReqTrx, currenGameBoardState.getIdelPlayerName()));

               final View gameStateView = this.flowEngine
                    .subFlow(new ViewBuilder(utxoTrxId));

               return new FlowResponce(gameStateView, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          } 
          catch (Exception e) {
               log.warn(requestBody + " [ERROR] " +e.toString());
               return new FlowResponce(e, utxoTrxId)
                    .toJsonEncodedString(jsonMarshallingService);
          }
     }

}
