package djmil.cordacheckers.gamestate;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import net.corda.v5.application.flows.CordaInject;
import net.corda.v5.application.flows.InitiatingFlow;
import net.corda.v5.application.flows.SubFlow;
import net.corda.v5.application.membership.MemberLookup;
import net.corda.v5.application.messaging.FlowMessaging;
import net.corda.v5.application.messaging.FlowSession;
import net.corda.v5.base.annotations.Suspendable;
import net.corda.v5.base.types.MemberX500Name;
import net.corda.v5.crypto.SecureHash;
import net.corda.v5.ledger.utxo.UtxoLedgerService;
import net.corda.v5.ledger.utxo.transaction.UtxoSignedTransaction;

@InitiatingFlow(protocol = "gamestate-commit")
public class CommitTrx implements SubFlow<SecureHash> {

    private final static Logger log = LoggerFactory.getLogger(CommitTrx.class);
    private final UtxoSignedTransaction utxTrxCandidate;
    private final MemberX500Name counterpartyName;
    private final MemberX500Name custodyName;

    public CommitTrx(UtxoSignedTransaction signedTransaction, MemberX500Name counterpartyName) {
        this.utxTrxCandidate = signedTransaction;
        this.counterpartyName = counterpartyName;
        this.custodyName = null;
    }

    public CommitTrx(UtxoSignedTransaction signedTransaction, MemberX500Name counterpartyName, MemberX500Name custodyName) {
        this.utxTrxCandidate = signedTransaction;
        this.counterpartyName = counterpartyName;
        this.custodyName = custodyName;
    }

    @CordaInject
    public UtxoLedgerService ledgerService;

    @CordaInject
    public FlowMessaging flowMessaging;

    @CordaInject
    public MemberLookup memberLookup;

    @Override
    @Suspendable
    public SecureHash call() {
        log.info("GameState commit started");

        /*
         * Calls the Corda provided finalise() function which gather signatures from the counterparty,
         * notarises the transaction and persists the transaction to each party's vault.
         */

        final FlowSession session = flowMessaging.initiateFlow(this.counterpartyName);
        List<FlowSession> sessionsList = new LinkedList<FlowSession>(Arrays.asList(session));

        if (custodyName != null) {
            System.out.println("we have a custoduan " + custodyName.getCommonName() );
            //sessionsList.add(flowMessaging.initiateFlow(custodyName));
        }

        final SecureHash trxId = ledgerService
            .finalize(this.utxTrxCandidate, sessionsList)
            .getTransaction()
            .getId();

        System.out.println("Trx commited "+trxId);

        log.info("GameState commit " +trxId);
        return trxId;
    }
}
