import { usePolling, doPushing } from '../hook/api';
import { useGamesGuideReducer, useGamesStateReducer, gamesGuideTemplate } from '../reducer/games';

export default function useGamesApi() {
    const [state, dispatchState] = useGamesStateReducer();
    const [guide, dispatchGuide] = useGamesGuideReducer();

    const useGamesPolling = (config) => {
        const onPolling = (isPolling) => dispatchGuide({ type: 'next', isPolling });
        const onSuccess = (games) => dispatchState({ type: 'next', games });

        usePolling('/api/game', { onPolling, onSuccess }, config.intervalMode(30));
    }

    const pushNewGame = ({ opponentName, opponentColor, board, message }) => {
        doPushing('/api/gameproposal', 'POST', { opponentName, opponentColor, board, message }, {
            onPushing: (isPushing) => dispatchGuide({ type: 'nextNewGame', isPushing }),
            onSuccess: (game) => {
                dispatchState({ type: 'add', game });
                dispatchGuide({ type: 'nextNewGame', ...gamesGuideTemplate.newGame });
            }
        })
    }

    const pushGameProposalAccept = ({ uuid }) => {
        doPushing(`/api/gameproposal/${uuid}/accept`, 'PUT', null, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && 'accept' }),
            onSuccess: (game) => dispatchState({ type: 'update', game })
        })
    }

    const pushGameProposalReject = ({ uuid }) => {
        doPushing(`/api/gameproposal/${uuid}/reject`, 'PUT', null, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && 'reject' }),
            onSuccess: (game) => dispatchState({ type: 'update', game })
        })
    }

    const pushGameProposalCancel = ({ uuid }) => {
        doPushing(`/api/gameproposal/${uuid}/cancel`, 'PUT', null, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && 'cancel' }),
            onSuccess: (game) => dispatchState({ type: 'update', game })
        })
    }

    const pushGameSurrender = ({ uuid }) => {
        doPushing(`/api/game/${uuid}/surrender`, 'PUT', null, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && 'surrender' }),
            onSuccess: (game) => dispatchState({ type: 'update', game })
        })
    }

    const pushGameDrawRequest = ({ uuid }) => {
        doPushing(`/api/game/${uuid}/drawreq`, 'PUT', null, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && 'draw_request' }),
            onSuccess: (game) => dispatchState({ type: 'update', game })
        })
    }

    const pushGameDrawAccept = ({ uuid }) => {
        doPushing(`/api/game/${uuid}/drawacc`, 'PUT', null, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && 'draw_accept' }),
            onSuccess: (game) => dispatchState({ type: 'update', game })
        })
    }

    const pushGameDrawReject = ({ uuid }) => {
        doPushing(`/api/game/${uuid}/drawrej`, 'PUT', null, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && 'draw_reject' }),
            onSuccess: (game) => dispatchState({ type: 'update', game })
        })
    }

    const pushGameMove = ({ uuid, move, message }) => {
        doPushing(`/api/game/${uuid}/move`, 'PUT', { move, message }, {
            onPushing: (isPushing) => dispatchGuide({ type: 'UUIDpushing', uuid, what: isPushing && {moveFrom: move[0], moveTo: move[1]} }),
            onSuccess: (game) => {
                dispatchState({ type: 'update', game });
                dispatchGuide({ type: 'UUIDmessage', uuid, message: ''});
            }
        })
    }

    return {
        state,
        guide,
        dispatchGuide,
        api: {
            useGamesPolling,
            pushNewGame,
            pushGameProposalAccept,
            pushGameProposalReject,
            pushGameProposalCancel,
            pushGameSurrender,
            pushGameDrawRequest,
            pushGameDrawAccept,
            pushGameDrawReject,
            pushGameMove
        }
    }
}