import React from 'react';

import { Color, Player, Board } from '../../components/Checkers';
import './GameBoard.css';

export default function GameBoard({ username, getGame, onStoneClick, onStoneMove }) {
  const [game, isPushing] = getGame();

  const myName = game.opponentName ? username : '';
  const opponentColor = Color.opposite(game.myColor);
  const flipBoard = (game.myColor === Color.black) ? true : null;

  const optionalOnStoneClick = (typeof onStoneClick !== 'function') ? null :
    (cellId) => onStoneClick(game.uuid, cellId);

  const optionalOnStoneMove = (typeof onStoneMove !== 'function' || isPushing) ? null :
    (move) =>  {
      if (move[0] !== move[1] && game.board[move[1]] === undefined)
        onStoneMove(game.uuid, move)
    };

  return (
    <div className='GameBoard'>
      <Player
        color={opponentColor || Color.black}
        name={game.opponentName}
      />
      <Board
        board={game.board}
        flip={flipBoard}
        prevMove={game.previousMove}
        currMove={[isPushing?.moveFrom, isPushing?.moveTo]}
        onStoneClick={optionalOnStoneClick}
        onStoneMove={optionalOnStoneMove}
      />
      <Player
        color={game.myColor || Color.white}
        name={myName}
      />
    </div>
  )
}