import { useReducer } from 'react';
import { nextState } from '../util/StateHelper';
import { defaultBoard } from '../components/Checkers';

/*
 * State
 */
const gameTemplate = {
  status: '',
  myColor: '',
  opponentName: '',

  board: null,
  moveNumber: 0,
  previousMove: [],

  message: '',
  uuid: ''
}

const gamesStateTemplate = [/* gameTemplate */];

function gamesStateReducer(state, action) {
  switch (action.type) {

    case 'next':
      return action.games;

    case 'add':
      return [
        ...state,
        nextState(gameTemplate, action.game, 'Game.create')
      ];

    case 'update':
      return state.map((game) =>
        game.uuid !== action.game.uuid ? game :
          nextState(gameTemplate, action.game, 'Game.update')
      );

    default:
      throw Error('GamesState: unknown action.type', action.type);
  }
}

export function useGamesStateReducer() {
  return useReducer(gamesStateReducer, gamesStateTemplate);
}

/*
 * Guide
 */
export const gamesGuideTemplate = {
  newGame: {
    opponentName: '',
    myColor: '',
    board: defaultBoard,
    message: '',
    isPushing: false
  },

  awaiting: {
    proposal: 0,
    active: 0
  },

  selectedUUID: {
    proposal: null,
    active: null,
    archive: null,
  },

  UUIDmessage: { // UUIDmessage[uuid] 
  },

  UUIDpushing: { // UUIDpushing[uuid]
  },

  isPolling: false,
};

function gamesGuideReducer(state, action) {
  switch (action.type) {

    case 'next':
      return nextState(state, action, 'GamesGuide');

    case 'sync':
      //console.log('sync');
      return {
        ...state,
        awaiting: calcAwating(action.gamesState)
      }

    case 'nextNewGame':
      return {
        ...state,
        newGame: nextState(state.newGame, action, 'GamesGuide.newGame')
      };

    case 'selectedUUID':
      return {
        ...state,
        selectedUUID: nextState(state.selectedUUID, action, 'GamesGuide.selectedUUID')
      };

    case 'UUIDmessage': {
      const next = { ...state };
      next.UUIDmessage[action.uuid] = action.message;
      return next;
    }

    case 'UUIDpushing': {
      const next = { ...state };
      next.UUIDpushing[action.uuid] = action.what
      return next;
    }

    default:
      throw Error('GamesGuide: unknown action.type: ' + action.type);
  }
}

//const uuidRegex = new RegExp("^[0-9a-f]{8}-[0-9a-f]{4}-[0-5][0-9a-f]{3}-[089ab][0-9a-f]{3}-[0-9a-f]{12}");

export function useGamesGuideReducer() {
  return useReducer(gamesGuideReducer, gamesGuideTemplate);
}

function calcAwating(games) {
  return games.reduce((awaiting, game) => {
    switch (game.status) {
      case 'GAME_PROPOSAL_WAIT_FOR_YOU':
        awaiting.proposal++;
        return awaiting;
      case 'GAME_BOARD_WAIT_FOR_YOU':
      case 'DRAW_REQUEST_WAIT_FOR_YOU':
        awaiting.active++;
        return awaiting;
      default:
        return awaiting;
    }
  }, structuredClone(gamesGuideTemplate.awaiting));
}